@@ -35,7 +35,6 @@ from .i18n import _
 ## should this last be changed so that may_unpair is used for all receivers? writing to _R.receiver_pairing doesn't seem right
 # re_pairs determines whether a receiver pairs by replacing existing pairings, default to False
 ## currently only one receiver is so marked - should there be more?
-# ex100_27mhz_wpid_fix enable workarounds for EX100 and possible other old 27Mhz receivers
 
 _DRIVER = ('hid-generic', 'generic-usb', 'logitech-djreceiver')
 
@@ -56,7 +55,8 @@ _unifying_receiver = lambda product_id: {
     'usb_interface': 2,
     'hid_driver': _DRIVER,  # noqa: F821
     'name': _('Unifying Receiver'),
-    'receiver_kind': 'unifying'
+    'receiver_kind': 'unifying',
+    'may_unpair': True
 }
 
 _nano_receiver = lambda product_id: {
@@ -112,7 +112,8 @@ _lenovo_receiver = lambda product_id: {
     'usb_interface': 1,
     'hid_driver': _DRIVER,  # noqa: F821
     'name': _('Nano Receiver'),
-    'receiver_kind': 'nano'
+    'receiver_kind': 'nano',
+    'may_unpair': False
 }
 
 _lightspeed_receiver = lambda product_id: {
@@ -120,7 +121,8 @@ _lightspeed_receiver = lambda product_id: {
     'product_id': product_id,
     'usb_interface': 2,
     'hid_driver': _DRIVER,  # noqa: F821
-    'name': _('Lightspeed Receiver')
+    'name': _('Lightspeed Receiver'),
+    'may_unpair': False
 }
 
 _ex100_receiver = lambda product_id: {
@@ -132,12 +134,12 @@ _ex100_receiver = lambda product_id: {
     'receiver_kind': '27Mhz',
     'max_devices': 4,
     'may_unpair': False,
-    're_pairs': True,
-    'ex100_27mhz_wpid_fix': True
+    're_pairs': True
 }
 
 # Receivers added here should also be listed in
 # share/solaar/io.github.pwr_solaar.solaar.metainfo.xml
+# Look in https://github.com/torvalds/linux/blob/master/drivers/hid/hid-ids.h
 
 # Bolt receivers (marked with the yellow lightning bolt logo)
 BOLT_RECEIVER_C548 = _bolt_receiver(0xc548)
@@ -157,7 +159,9 @@ NANO_RECEIVER_C526 = _nano_receiver(0xc526)
 NANO_RECEIVER_C52E = _nano_receiver_no_unpair(0xc52e)
 NANO_RECEIVER_C531 = _nano_receiver(0xc531)
 NANO_RECEIVER_C534 = _nano_receiver_max2(0xc534)
+NANO_RECEIVER_C535 = _nano_receiver(0xc535)  # branded as Dell
 NANO_RECEIVER_C537 = _nano_receiver(0xc537)
+# NANO_RECEIVER_C542 = _nano_receiver(0xc542) # does not use HID++
 NANO_RECEIVER_6042 = _lenovo_receiver(0x6042)
 
 # Lightspeed receivers (usually sold with gaming devices)
@@ -170,7 +174,9 @@ LIGHTSPEED_RECEIVER_C545 = _lightspeed_receiver(0xc545)
 LIGHTSPEED_RECEIVER_C547 = _lightspeed_receiver(0xc547)
 
 # EX100 old style receiver pre-unifying protocol
+# EX100_27MHZ_RECEIVER_C50C = _ex100_receiver(0xc50C)  # in hid/hid-ids.h
 EX100_27MHZ_RECEIVER_C517 = _ex100_receiver(0xc517)
+# EX100_27MHZ_RECEIVER_C51B = _ex100_receiver(0xc51B)  # in hid/hid-ids.h
 
 ALL = (
     BOLT_RECEIVER_C548,
@@ -186,7 +192,9 @@ ALL = (
     NANO_RECEIVER_C52E,
     NANO_RECEIVER_C531,
     NANO_RECEIVER_C534,
+    NANO_RECEIVER_C535,
     NANO_RECEIVER_C537,
+    #    NANO_RECEIVER_C542,  # does not use HID++
     NANO_RECEIVER_6042,
     LIGHTSPEED_RECEIVER_C539,
     LIGHTSPEED_RECEIVER_C53A,
@@ -226,7 +234,7 @@ def other_device_check(bus_id, vendor_id, product_id):
         if (product_id >= 0xC07D and product_id <= 0xC094 or product_id >= 0xC32B and product_id <= 0xC344):
             return _wired_device(product_id, 2)
     elif bus_id == 0x5:  # Bluetooth
-        if (product_id >= 0xB012 and product_id <= 0xB0FF or product_id >= 0xB32A and product_id <= 0xB3FF):
+        if (product_id >= 0xB012 and product_id <= 0xB0FF or product_id >= 0xB317 and product_id <= 0xB3FF):
             return _bt_device(product_id)
 
 
